import {Injectable} from "@angular/core";
import {CookieService} from "ngx-cookie-service";
import {HttpClient} from "@angular/common/http";
import {catchError, tap, throwError} from "rxjs";
import {iAccessToken} from "../model/iacces-token.model";
import {UtilsService} from "./utils.service";
import {toMs} from "ms-typescript";

@Injectable({
  providedIn: 'root'
})
export class RefreshService {

  constructor(
    private readonly cookieService: CookieService,
    private readonly http: HttpClient,
  ) {}

  private _getExpirationForCookie() {
    const today = new Date();
    today.setTime(today.getTime() + (60 * 1000));
    return today;
  }

  private _refreshToken(uri: string, token: string) {
    return this.http.get<iAccessToken>(
      uri + 'accounts/auth/refresh',
      {
        headers: {
          authorization: 'Bearer ' + token
        }
      }
    )
  }

  private _cookieSetNewToken(uri: string, token: string) {
    this._refreshToken(uri, token).pipe(
      tap((resp: iAccessToken) =>
        this.cookieService.set('_ustkv', resp.access_token, {
          expires: this._getExpirationForCookie()
        })
      ),
      catchError(err => {
        UtilsService.clean(this.cookieService)
        return throwError(err);
      })
    ).subscribe();
  }

  refreshToken(uri: string, token: string) {
    setInterval(() => {
      this._cookieSetNewToken(uri, token);
    }, toMs('10m'));
  }
}
