import { Injectable } from '@angular/core';
import {CookieService} from 'ngx-cookie-service';
import {jwtDecode} from 'jwt-decode';
import {HttpClient} from '@angular/common/http';
import {catchError, Observable, tap, throwError} from 'rxjs';
import {iPayload} from "../model/ipayload.model";
import {UtilsService} from "./utils.service";
import {RefreshService} from "./refresh.service";

@Injectable({
  providedIn: 'root'
})
export class UserCookieService<T> {
  user!: T;
  err: string = '';

  constructor(
    private readonly cookieService: CookieService,
    private readonly refreshService: RefreshService,
    private readonly http: HttpClient
  ) {}

  private _getApiCallForUser<T>(uri: string, id: string, token: string): Observable<T> {
    return this.http.get<T>(
      uri + 'accounts/user/' + id,
      {
        headers: {
          authorization: 'Bearer ' + token
        }
      }
    );
  }

  isValidUser(uri: string): Observable<T> {
    if (localStorage.getItem('ustkv') == null) {
      UtilsService.clean(this.cookieService);
    }

    const token: string = this.cookieService.get('_ustkv');

    if (token.length == 0) {
      this.refreshService.refreshToken(uri, localStorage.getItem('ustkv')!);
    }

    const payload: iPayload = jwtDecode(token);

    return this._getApiCallForUser(uri, payload.id, token)
      .pipe(
        // @ts-ignore
        tap((user: T) => this.user = user),
        catchError((err) => {
          UtilsService.clean(this.cookieService)
          return throwError(err);
        })
      );
  }
}
